! This file is part of HiggsBounds
!  -KW
!******************************************************************
module theo_manip
!******************************************************************
 !use S95tables_type1
 use usefulbits, only : ndat,np,Hneut,Hplus,theo,tevR,tevextras,pdesc
 
 implicit none

 type(tevextras) :: tevS(1)

 contains

 !******************************************************************
 subroutine complete_theo
 !decides what has to be done to the input and calls the appropriate 
 !subroutines
 !calculates theo(jj)%inTEVrange_hj(i),theo(jj)%inLEPrange_hj(i)
 !******************************************************************
  use usefulbits, only : whichanalyses,whichinput,ndat,&
          &              TEVrange_Mhmin,TEVrange_Mhmax,LEPrange_Mhmin,LEPrange_Mhmax
  implicit none
  !--------------------------------------internal
  integer :: i,j,jj
  !----------------------------------------------

  if(np(Hneut)>0)then !none if this is needed for the charged Higgs sector yet
   do jj=1,ndat 
    do i=1,np(Hneut)
! we want to include contributions to fact(j) from Higgs 
! with masses above TEVt1Mhmim-max_tev_deltaMh but less than TEVt1Mhmax+max_tev_deltaMh
     theo(jj)%inTEVrange_hj(i)=(theo(jj)%particle(Hneut)%M(i).gt.TEVrange_Mhmin) &
                          .and.(theo(jj)%particle(Hneut)%M(i).lt.TEVrange_Mhmax)

     theo(jj)%inLEPrange_hj(i)=(theo(jj)%particle(Hneut)%M(i).gt.LEPrange_Mhmin) &
                          .and.(theo(jj)%particle(Hneut)%M(i).lt.LEPrange_Mhmax)
    enddo
   enddo

   select case(whichinput)        
   case('effC')  
    call csratios_from_g2
    call cp_from_g2
    call br_from_g2
   case('SLHA')
    call csratios_from_g2
    call cp_from_g2
   case('hadr','part')
   case default
    stop'error in subroutine complete_theo (1)' 
   end select               
    
  
   do jj=1,ndat     ! filling the other half of XS_hjhi_ratio
    do j=2,np(Hneut)
     do i=1,j-1          
      theo(jj)%lep%XS_hjhi_ratio(i,j) = theo(jj)%lep%XS_hjhi_ratio(j,i)               
     enddo
    enddo
   enddo  
  endif

  call check_dataset !involves the charged Higgs sector
 
  if(np(Hneut)>0)then !none if this is needed for the charged Higgs sector yet
   select case(whichanalyses)
   case('onlyT','LandT','singH','onlyP') ! everything which involves Tevatron tables

    call fill_theo_SM ! n.b. there's no LEP SM cross sections at the moment
  
     select case(whichinput)
     case('part','effC','SLHA') ! everything except option 'hadr', where had XS ratios are inputted directly
      call XS_from_tevR
     case('hadr')
     case default
      stop'error in subroutine complete_theo (2)'  
     end select
   case('onlyL')
   case default       
    stop'error in subroutine complete_theo (3)'             
   end select   
  endif


 end subroutine complete_theo

 !****************************************************************** 
 subroutine csratios_from_g2 
 ! uses the effective couplings contained in g2 to calculate 
 ! partonic cross section ratios, some hadronic cross section ratios
 !*****************************************************************
  use usefulbits, only : g2,ndat,vsmall
  implicit none
  !--------------------------------------internal
  integer :: i,jj
  double precision :: SM_ZZ_contrib_to_VBF,SM_WW_contrib_to_VBF
  !---------------------------------------------

  if(np(Hneut)<1)stop'error in csratios_from_g2  (np(Hneut))'

! relative contributuion of WW- and ZZ-fusion to VBF (in LO) for
! p p-bar collisions at SqrtS=1.96 TeV (calcuated by T. Figy with VBF@NLO):
   SM_ZZ_contrib_to_VBF=0.23D0
   SM_WW_contrib_to_VBF=0.77D0

   do jj=1,ndat
    do i=1,np(Hneut)
                
      theo(jj)%lep%XS_hjZ_ratio(i)      =  g2(jj)%hjZZ(i) 
      theo(jj)%lep%XS_bbhj_ratio(i)     =  g2(jj)%hjbb_s(i)+g2(jj)%hjbb_p(i)!nb tev tables at the moment can not mixed CP Higgs
      theo(jj)%lep%XS_tautauhj_ratio(i) =  g2(jj)%hjtautau_s(i)+g2(jj)%hjtautau_p(i)!nb tev tables at the moment can not mixed CP Higgs

      tevR(jj)%bg_hjb(i)             =   g2(jj)%hjbb_s(i)+g2(jj)%hjbb_p(i)
      theo(jj)%tev%XS_vbf_ratio(i)   =   g2(jj)%hjWW(i)*SM_WW_contrib_to_VBF &        
                               &       + g2(jj)%hjZZ(i)*SM_ZZ_contrib_to_VBF
      theo(jj)%tev%XS_tthj_ratio(i)  =   g2(jj)%hjtoptop_s(i)+g2(jj)%hjtoptop_p(i) !nb tev tables at the moment can only use CP even Higgs
           
      tevR(jj)%qq_hjWp(:,i)      = g2(jj)%hjWW(i)
      tevR(jj)%qq_hjWm(:,i)      = g2(jj)%hjWW(i) 
      tevR(jj)%gg_hj(i)          = g2(jj)%hjgg(i)
      tevR(jj)%qq_hj(5,i)         = g2(jj)%hjbb_s(i)+g2(jj)%hjbb_p(i)        
      tevR(jj)%qq_hjZ(:,i)       = g2(jj)%hjZZ(i)           
        
    enddo 

    theo(jj)%lep%XS_hjhi_ratio=g2(jj)%hjhiZ! note only half of XS_hjhi_ratio is filled here

   enddo

 end subroutine csratios_from_g2

 !****************************************************************** 
 subroutine cp_from_g2 
 ! uses the effective couplings contained in g2 to calculate 
 ! cp of neutral higgs
 !*****************************************************************
  use usefulbits, only : g2,ndat,vsmall
  implicit none
  !--------------------------------------internal
  integer :: i,jj
  double precision :: max_hjff_s,max_hjff_p
  !---------------------------------------------

  if(np(Hneut)<1)stop'error in cp_from_g2  (np(Hneut))'

   do jj=1,ndat
 
    do i=1,np(Hneut)
     max_hjff_s=max(g2(jj)%hjss_s(i),g2(jj)%hjcc_s(i),g2(jj)%hjbb_s(i),g2(jj)%hjtoptop_s(i),g2(jj)%hjtautau_s(i))

     max_hjff_p=max(g2(jj)%hjss_p(i),g2(jj)%hjcc_p(i),g2(jj)%hjbb_p(i),g2(jj)%hjtoptop_p(i),g2(jj)%hjtautau_p(i))

     if(     max_hjff_p .lt. vsmall )then !CP even
      theo(jj)%CP_value(i) =  1
     elseif( max_hjff_s .lt. vsmall )then !CP odd
      theo(jj)%CP_value(i) = -1
     else                              !mixed CP
      theo(jj)%CP_value(i) =  0
     endif
    enddo 
 
   enddo

 end subroutine cp_from_g2

 !****************************************************************** 
 subroutine br_from_g2 
 ! uses the effective couplings contained in g2 to calculate 
 ! branching ratios
 !*****************************************************************
  use theory_BRfunctions
  use usefulbits, only : g2,ndat,ms,mc,mbmb,mtau,vsmall
  implicit none
  !--------------------------------------internal
  integer :: i,jj
  double precision :: Mhi,GammaRat
  !---------------------------------------------

  if(np(Hneut)<1)stop'error in br_from_g2 (np(Hneut))'

   do jj=1,ndat
    do i=1,np(Hneut)
      Mhi=theo(jj)%particle(Hneut)%M(i)
                
      if(theo(jj)%inTEVrange_hj(i).or.theo(jj)%inLEPrange_hj(i))then
   
       GammaRat=theo(jj)%particle(Hneut)%GammaTot(i)/BRSM_GammaTot(Mhi)  
             
       if(theo(jj)%particle(Hneut)%GammaTot(i).gt.0.0D0)then
        theo(jj)%BR_hjss(i)    = ( g2(jj)%hjss_s(i)    +g2(jj)%hjss_p(i)    *invbsq(ms,  Mhi) ) *BRSM_Hss(Mhi)     /GammaRat
        theo(jj)%BR_hjcc(i)    = ( g2(jj)%hjcc_s(i)    +g2(jj)%hjcc_p(i)    *invbsq(mc,  Mhi) ) *BRSM_Hcc(Mhi)     /GammaRat
        theo(jj)%BR_hjbb(i)    = ( g2(jj)%hjbb_s(i)    +g2(jj)%hjbb_p(i)    *invbsq(mbmb,Mhi) ) *BRSM_Hbb(Mhi)     /GammaRat
        theo(jj)%BR_hjtautau(i)= ( g2(jj)%hjtautau_s(i)+g2(jj)%hjtautau_p(i)*invbsq(mtau,Mhi) ) *BRSM_Htautau(Mhi) /GammaRat

        theo(jj)%BR_hjWW(i)    = g2(jj)%hjWW(i)    *BRSM_HWW(Mhi)    /GammaRat 
        theo(jj)%BR_hjZZ(i)    = g2(jj)%hjZZ(i)    *BRSM_HZZ(Mhi)    /GammaRat 
        theo(jj)%BR_hjZga(i)   = g2(jj)%hjZga(i)   *BRSM_HZga(Mhi)   /GammaRat 
        theo(jj)%BR_hjgaga(i)  = g2(jj)%hjgaga(i)  *BRSM_Hgaga(Mhi)  /GammaRat  
        theo(jj)%BR_hjgg(i)    = g2(jj)%hjgg(i)    *BRSM_Hgg(Mhi)    /GammaRat  
       else

        write(*,*)'at jj=',jj,'i=',i
        write(*,*)'total decay width is less than or equal to zero:',theo(jj)%particle(Hneut)%GammaTot(i)
        stop'total decay width is less than or equal to zero'
       endif     
      else
        theo(jj)%BR_hjss(i)    = 0.0D0
        theo(jj)%BR_hjcc(i)    = 0.0D0       
        theo(jj)%BR_hjbb(i)    = 0.0D0
        theo(jj)%BR_hjtautau(i)= 0.0D0 
        theo(jj)%BR_hjWW(i)    = 0.0D0 
        theo(jj)%BR_hjZZ(i)    = 0.0D0 
        theo(jj)%BR_hjZga(i)   = 0.0D0
        theo(jj)%BR_hjgaga(i)  = 0.0D0
        theo(jj)%BR_hjgg(i)    = 0.0D0
      endif            
        
    enddo 

   enddo

 end subroutine br_from_g2

 function invbsq(mf,mh)
  implicit none
  double precision,intent(in) :: mf,mh
  double precision :: invbsq
  if(mh>2.0D0*mf)then 
   invbsq=1.0D0/(1.0D0-4.0D0*(mf/mh)**2.0D0)
  else
   invbsq=0.0D0
  endif
 end function invbsq
 !*****************************************************************
 subroutine check_dataset
 ! checks each parameter point to determine whether the Higgs masses
 ! and branching ratios make sense
 ! Sets theo(jj)%gooddataset accordingly
 !*****************************************************************
  use usefulbits, only : theo,ndat,debug,np
  implicit none
  !--------------------------------------internal
  integer :: jj,x
  double precision :: testsumBR,testsumBR_t
  double precision,allocatable :: testBR(:)
  double precision :: fuzziness
  !---------------------------------------------
  fuzziness = 0.001D0
  !fuzziness = 100.0D0 ; write(*,*)'WARNING: fuzziness factor is far too high'

  testsumBR =0.0D0
  testsumBR_t  =0.0D0

  if(np(Hneut)>0)then
   allocate(testBR(np(Hneut)))

   ! testing to see if the dataset is ok
   do jj=1,ndat   
    testBR     =         theo(jj)%BR_hjss              &
            &      +     theo(jj)%BR_hjcc              &
            &      +     theo(jj)%BR_hjbb              &
            &      +     theo(jj)%BR_hjtautau          &
            &      +     theo(jj)%BR_hjWW              &
            &      +     theo(jj)%BR_hjZZ              &
            &      +     theo(jj)%BR_hjZga             &
            &      +     theo(jj)%BR_hjgaga            &
            &      + sum(theo(jj)%BR_hjhihi,dim=2)
    testsumBR  = maxval(  testBR  ) 
    if( testsumBR   .gt.   1.0D0+fuzziness   )then     
     if(debug)write(*,*) 'warning: sum of BR for '//trim(adjustl(pdesc(Hneut)%long))//' at line number=',jj,'is',testsumBR
    endif
   enddo
   deallocate(testBR)
  endif 

  if(np(Hplus)>0)then
   allocate(testBR(np(Hplus)))

   do jj=1,ndat 
    testBR      =        theo(jj)%BR_Hpjcs             &
            &      +     theo(jj)%BR_Hpjcb             &
            &      +     theo(jj)%BR_Hpjtaunu          
    testsumBR = maxval(  testBR ) 


    testsumBR_t =        theo(jj)%BR_tWpb              &
            &      + sum(theo(jj)%BR_tHpjb,dim=1)

    if( testsumBR  .gt.   1.0D0+fuzziness   )then     
     if(debug)write(*,*) 'warning: sum of BR for '//trim(adjustl(pdesc(Hplus)%long))//' at line number=',jj,'is',testsumBR
    elseif( testsumBR_t    .gt.   1.0D0+fuzziness   )then     
     if(debug)write(*,*) 'warning: sum of BR for the top quark at jj=',jj,'is',testsumBR_t  
    endif 

   enddo
   deallocate(testBR)
  endif

  do jj=1,ndat
     theo(jj)%gooddataset=.True.      
  enddo 

  do x=1,ubound(np,dim=1)
   if(np(x)>0)then
    do jj=1,ndat 
     if(     minval(theo(jj)%particle(x)%M).lt.0.0D0)then
      theo(jj)%gooddataset=.False.
      if(debug)write(*,*) 'warning: negative mass for '//trim(adjustl(pdesc(x)%long))//' at line number=',jj,theo(jj)%particle(x)%M

    !elseif( testsumBR_hj  .gt.   (1.0D0+fuzziness)   )then !i.e. branching ratios for one of the Higgs add up to more than 1+fuzziness
    ! !theo(jj)%gooddataset=.False. 
     elseif( .not. (sum(theo(jj)%particle(x)%M).ge.0.0D0) )then   
      theo(jj)%gooddataset=.False.
      write(*,*) 'warning: mass is NaN for '//trim(adjustl(pdesc(x)%long))//' at line number=',jj,theo(jj)%particle(x)%M

     elseif(     minval(theo(jj)%particle(x)%GammaTot).lt.0.0D0)then
      theo(jj)%gooddataset=.False.
      if(debug)write(*,*) 'warning: negative total decay width for '//trim(adjustl(pdesc(x)%long))// &
                        & ' at line number=',jj,theo(jj)%particle(x)%GammaTot

    !elseif( testsumBR_hj  .gt.   (1.0D0+fuzziness)   )then !i.e. branching ratios for one of the Higgs add up to more than 1+fuzziness
    ! !theo(jj)%gooddataset=.False. 
     elseif( .not. (sum(theo(jj)%particle(x)%GammaTot).ge.0.0D0) )then   
      theo(jj)%gooddataset=.False.
      if(debug)write(*,*) 'warning: total decay width is NaN for '//trim(adjustl(pdesc(x)%long))// &
                        & ' at line number=',jj,theo(jj)%particle(x)%GammaTot

     endif 

    enddo
   endif
  enddo

 end subroutine check_dataset

 !*****************************************************************
 subroutine fill_theo_SM
 ! fills the Standard Model part of theo
 ! We do this here to save computational time - these  quantities will be
 ! needed a few times in subroutine calcfact_t1, so don't want to calculate them each time
 !************************************************************
  use theory_BRfunctions
  use theory_XS_SM_functions
  use usefulbits, only : theo,ndat

  implicit none
  !--------------------------------------internal
  integer :: i,n
  double precision :: Mhi
  !----------------------------------------------
  
  if(np(Hneut)<1)stop'error in subroutine fill_theo_SM (np(Hneut))'

   do n=1,ndat        
    if(theo(n)%gooddataset) then             
    do i=1,np(Hneut)
  
     Mhi=theo(n)%particle(Hneut)%M(i)    
    
     if( .not.theo(n)%inTEVrange_hj(i) )then      
       theo(n)%tev%XS_HW_SM(i) = 0.0D0
       theo(n)%tev%XS_H_SM(i)  = 0.0D0
       theo(n)%tev%XS_HZ_SM(i) = 0.0D0
       theo(n)%tev%XS_vbf_SM(i)= 0.0D0
       theo(n)%tev%XS_ttH_SM(i)= 0.0D0
       !theo(n)%tev%XS_H_SM_9713(i)  = 0.0D0
       !theo(n)%tev%XS_H_SM_9674(i)  = 0.0D0
      
       theo(n)%tev%XS_Hb_SM(i)    = 0.0D0
       theo(n)%tev%XS_Hb_c1_SM(i) = 0.0D0
       theo(n)%tev%XS_Hb_c2_SM(i) = 0.0D0
       theo(n)%tev%XS_Hb_c3_SM(i) = 0.0D0
          
       theo(n)%BR_Hbb_SM(i)     = -1.0D0
       theo(n)%BR_Htautau_SM(i) = -1.0D0
       theo(n)%BR_HWW_SM(i)     = -1.0D0
       theo(n)%BR_HZZ_SM(i)     = -1.0D0
       theo(n)%BR_HZga_SM(i)    = -1.0D0
       theo(n)%BR_Hgaga_SM(i)   = -1.0D0
       theo(n)%BR_Hjets_SM(i)   = -1.0D0
       theo(n)%GammaTot_SM(i)   = -1.0D0         
     else
      
       theo(n)%BR_HWW_SM(i)    = BRSM_HWW(Mhi) 
       theo(n)%BR_HZZ_SM(i)    = BRSM_HZZ(Mhi)    
       theo(n)%BR_Hbb_SM(i)    = BRSM_Hbb(Mhi)
       theo(n)%BR_Htautau_SM(i)= BRSM_Htautau(Mhi)
       theo(n)%BR_HZga_SM(i)   = BRSM_HZga(Mhi)
       theo(n)%BR_Hgaga_SM(i)  = BRSM_Hgaga(Mhi)
       theo(n)%BR_Hjets_SM(i)  = BRSM_Hss(Mhi)+BRSM_Hcc(Mhi)+BRSM_Hbb(Mhi)+BRSM_Hgg(Mhi)
       theo(n)%GammaTot_SM(i)  = BRSM_GammaTot(Mhi)         
         
       theo(n)%tev%XS_HZ_SM(i) = XS_qq_HZ_SM(Mhi)   
       theo(n)%tev%XS_HW_SM(i) = XS_qq_HW_SM(Mhi)
       theo(n)%tev%XS_H_SM(i)  = XS_gg_H_SM(Mhi)+XS_bb_H_SM(Mhi)  
       theo(n)%tev%XS_vbf_SM(i)= XS_vbf_SM(Mhi)
       theo(n)%tev%XS_ttH_SM(i)= XS_ttH_SM(Mhi) 
       !theo(n)%tev%XS_H_SM_9713(i)  = XS_gg_H_SM_9713(Mhi)+XS_bb_H_SM(Mhi)   
       !theo(n)%tev%XS_H_SM_9674(i)  = XS_gg_H_SM_9674(Mhi)+XS_bb_H_SM(Mhi)   
           
       theo(n)%tev%XS_Hb_SM(i)    = XS_bg_Hb_SM(Mhi) 
       theo(n)%tev%XS_Hb_c1_SM(i) = XS_bg_Hb_c1_SM(Mhi) 
       theo(n)%tev%XS_Hb_c2_SM(i) = XS_bg_Hb_c2_SM(Mhi)    
       theo(n)%tev%XS_Hb_c3_SM(i) = XS_bg_Hb_c3_SM(Mhi)              
     endif  
    enddo
    endif                     
   enddo        
      
 end subroutine fill_theo_SM

 !************************************************************
 subroutine XS_from_tevR
 ! turn partonic cross section ratios in to hadronic cross section
 ! ratios
 ! Subroutine is complicated by the fact that if e.g. all
 ! the tevR(n)%qq_hjW partonic cross section ratios are equal,
 ! just want to use this value for the hadronic cross section
 ! ratio and not lose any accuracy by combining with the tevS
 !************************************************************
  use usefulbits, only : ndat, &
               & allocate_tevextras_parts,deallocate_tevextras_parts

  implicit none
  !--------------------------------------internal
  integer :: n,i
  logical :: simple_tevR
  !----------------------------------------------

  if(np(Hneut)<1)stop'error in subroutine XS_from_tevR (np(Hneut))'

    call allocate_tevextras_parts(tevS) 
  
    do n=1,ndat        
     if(theo(n)%gooddataset) then             
      do i=1,np(Hneut)

       if( theo(n)%inTEVrange_hj(i) )then         

        call fill_tevS(i,theo(n)%particle(Hneut)%M(i))      
        
        !this if loop is here to make sure tevR(n)%qq_hjWp(1,i).eq.0.0D0 is taken care of
        if(tevR(n)%qq_hjWp(1,i).eq.0.0D0)then
         simple_tevR=.False.
        elseif( (( sum(abs( tevR(n)%qq_hjWp(:,i) - tevR(n)%qq_hjWp(1,i)))   &
    &            + sum(abs( tevR(n)%qq_hjWm(:,i) - tevR(n)%qq_hjWp(1,i))) )/tevR(n)%qq_hjWp(1,i)) .lt. 1.0D-5 )then
         simple_tevR=.True.
        else
         simple_tevR=.False.
        endif

        if(simple_tevR)then
          theo(n)%tev%XS_hjW_ratio(i)=tevR(n)%qq_hjWp(1,i)
        else
          theo(n)%tev%XS_hjW_ratio(i)=                          &
           &   sum( tevR(n)%qq_hjWp(:,i)*tevS(1)%qq_hjWp(:,i) ) &
           & + sum( tevR(n)%qq_hjWm(:,i)*tevS(1)%qq_hjWm(:,i) )
        endif
    
        theo(n)%tev%XS_hj_ratio(i)=                         &
         &        tevR(n)%gg_hj(i)    *tevS(1)%gg_hj(i)     &
         & + sum( tevR(n)%qq_hj(:,i)  *tevS(1)%qq_hj(:,i)   )  
   
        !this if loop is here to make sure tevR(n)%qq_hjZ(1,i).eq.0.0D0 is taken care of
        if(tevR(n)%qq_hjZ(1,i).eq.0.0D0)then
         simple_tevR=.False.
        elseif( ( (abs(sum( tevR(n)%qq_hjZ(:,i) - tevR(n)%qq_hjZ(1,i)))/tevR(n)%qq_hjZ(1,i)).lt. 1.0D-5) &
    &      .and. (               tevS(1)%gg_hjZ(i).le.0.0D0                  ) )then
         simple_tevR=.True.
        else
         simple_tevR=.False.
        endif

        if(  simple_tevR  )then    
          theo(n)%tev%XS_hjZ_ratio(i)=  tevR(n)%qq_hjZ(1,i)
        else
          theo(n)%tev%XS_hjZ_ratio(i)=                     &
           &        tevR(n)%gg_hjZ(i)   *tevS(1)%gg_hjZ(i)    &
           & + sum( tevR(n)%qq_hjZ(:,i) *tevS(1)%qq_hjZ(:,i)  )
        endif

        theo(n)%tev%XS_hjb_ratio(i)= tevR(n)%bg_hjb(i)
       else
        theo(n)%tev%XS_hjW_ratio(i)= 0.0D0       
        theo(n)%tev%XS_hj_ratio(i) = 0.0D0  
        theo(n)%tev%XS_hjZ_ratio(i)= 0.0D0
        theo(n)%tev%XS_hjb_ratio(i)= 0.0D0
       endif  
      enddo
     endif                     
    enddo            
    
    call deallocate_tevextras_parts(tevS)

 end subroutine XS_from_tevR
 !******************************************************************  
 subroutine fill_tevS(j,Mhj)
 ! fills the elements of tevS using the functions in module theory_tevSfunctions
 !************************************************************
  use theory_tevSfunctions
  implicit none
  !--------------------------------------internal
  integer :: j
  double precision :: Mhj
  !----------------------------------------------
  
  tevS(1)%qq_hjWp(1,j)=tev_rHWpm_udb(Mhj)
  tevS(1)%qq_hjWp(2,j)=tev_rHWpm_csb(Mhj) 
   
  tevS(1)%qq_hjWm(1,j)=tev_rHWpm_dub(Mhj)
  tevS(1)%qq_hjWm(2,j)=tev_rHWpm_scb(Mhj)
  
  tevS(1)%gg_hj(j)=tev_rH_gg(Mhj)
  
  tevS(1)%qq_hj(1,j)=0.0D0
  tevS(1)%qq_hj(2,j)=0.0D0
  tevS(1)%qq_hj(3,j)=0.0D0
  tevS(1)%qq_hj(4,j)=0.0D0
  tevS(1)%qq_hj(5,j)=tev_rH_bb(Mhj)
  
  tevS(1)%gg_hjZ(j)=0.0D0
  
  tevS(1)%qq_hjZ(1,j)=tev_rHZ_ddb(Mhj)
  tevS(1)%qq_hjZ(2,j)=tev_rHZ_uub(Mhj)
  tevS(1)%qq_hjZ(3,j)=tev_rHZ_ssb(Mhj)
  tevS(1)%qq_hjZ(4,j)=tev_rHZ_ccb(Mhj)
  tevS(1)%qq_hjZ(5,j)=tev_rHZ_bbb(Mhj)
      

  if(abs(sum(tevS(1)%qq_hjWp(:,j))+sum(tevS(1)%qq_hjWm(:,j)) - 1.0D0) .gt. 1.0D-2)then 
   stop 'error in fill_tevS (a)'
  elseif(abs(tevS(1)%gg_hj(j)+sum(tevS(1)%qq_hj(:,j)) - 1.0D0) .gt. 1.0D-2)then
   stop 'error in fill_tevS (b)'
  elseif(abs(tevS(1)%gg_hjZ(j)+sum(tevS(1)%qq_hjZ(:,j)) - 1.0D0) .gt. 1.0D-2)then
   stop 'error in fill_tevS (c)'
  endif 

 end subroutine fill_tevS 
 !******************************************************************  
end module theo_manip
!******************************************************************
